

class Device:
    MODEL_MS300B    = "MS300B"
    MODEL_MS500A    = "MS500A"
    MODEL_MS700     = "MS700"


    def __init__(self,model:str):
        self._model = model


    def setRelay(self, relayId: int, stt: bool) -> None:
        raise NotImplementedError("setRelay not implemented")


    def setRedLed(self, stt: bool) -> None:
        raise NotImplementedError("setRedLed not implemented")


    def setGreenLed(self, stt: bool) -> None:
        raise NotImplementedError("setGreenLed not implemented")


    def readInEvent(self, inputId: int) -> int:
        raise NotImplementedError("readInEvent not implemented")

    def readInValue(self, inputId: int) -> int:
        raise NotImplementedError("readInValue not implemented")

    def getOutState(self, outputId: int ) -> int:
        raise NotImplementedError("getOutState not implemented")

    def getIoMode(self, inputId: int) -> str:
        raise NotImplementedError("getIoMode not implemented")

    def getSipPcmc(self) -> str:
        """
        Get the name of the SIP PCM capture device (microphone)
        """
        raise NotImplementedError("getSipPcmc not implemented")


    def getSipPcmp(self) -> str:
        """
        Get the name of the SIP PCM playback device (audio out)
        """
        raise NotImplementedError("getSipPcmp not implemented")


    def getModel(self) -> str:
        """
        Get the device model
        """
        raise NotImplementedError("getModel not implemented")

    def getRtpBgmCh(self) -> str:
        """
        Get the alsa channel for RTP BGM
        """
        return self._model


    def getRtpHpnCh(self) -> str:
        """
        Get the alsa channel for RTP HPN
        """
        raise NotImplementedError("getRtpHpnCh not implemented")

    def loadMixers(self) -> None:
        raise NotImplementedError("setupMixers not implemented")

#    def handleTcpCommand(self, msg: str) -> str:
#        """
#        Handle one EPIC command received over TCP
#        msg - the command, the terminating \\r\\n
#        """
#        raise NotImplementedError("handleTcpCommand not implemented")


#    def handleSerialMsg(self, msg: str) -> str:
#        """
#        Handle one serial message.
#        msg - serial message,
#              already stripped of terminating characters
#        returns: TBD
#        """
#        raise NotImplementedError("handleSerialMsg not implemented")


    def shutdown(self):
        """
        Shutdown operation of the serial GW and disconnect
        from the serial port
        """
        raise NotImplementedError("shutdown not implemented")

    def setExclusiveInput(self, lineId: int) -> None:
        """
        Set an exclusive input (1..6) or disable it with 0
        """
        raise NotImplementedError("setExclusiveInput not implemented")

    def setMainVolume(self, volume: int) -> None:
        """
        Sets the main volume output
        volume - the volume, from 0 (min) to 100 (max)
        """
        raise NotImplementedError("setMainVolume not implemented")

    def reloadSettings(self) -> None:
        """
        Reloads device specific settings on demand.
        Actual behaviour is device dependent.
        """
        raise NotImplementedError("reloadSettings not implemented")

    def identifyXd(self) -> None:
        """
        Put the XD receiver into identify mode
        """
        raise NotImplementedError("identifyXd not implemented")

    def activateDisplay(self) -> None:
        """
        Activate (or reset the timeout of) the display
        """
        raise NotImplementedError("activateDisplay not implemented")

    def muteAudioOutputs(self) -> None:
        """
        Mute all audio outputs
        """
        raise NotImplementedError("muteAudioOutputs not implemented")

    def unmuteAudioOutputs(self) -> None:
        """
        Unmute all audio outputs
        """
        raise NotImplementedError("unmuteAudioOutputs not implemented")

#    def writeSideChannel(self, data: str) -> None:
#        """
#        Write data to the sidechannel of the serial port
#        data - string to write into the serial port, regardless of
#               the operation mode.
#        """
#        raise NotImplementedError("writeSideChannel not implemented")
