
from device import Device
from ms500 import DeviceMs500
from ms700 import DeviceMs700


def _readModel() -> str:
    """
    Read the device model from the firmware rootfs
    """
    with open("/barix/apps/epic-sip/MODEL") as fIn:
        return fIn.readline().strip()


class DeviceFactory:
    _instance = None

    @classmethod
    def createDevice(cls) -> Device :
        """
        Sets up the device and eturns the newly created instance
        The instance can later be accessed using instance()
        """
        m   = _readModel()
        if m==Device.MODEL_MS300B or m==Device.MODEL_MS500A:
            cls._instance  = DeviceMs500(m)
        elif m==Device.MODEL_MS700:
            cls._instance  = DeviceMs700(m)
        else:
            raise Exception("device not supported: " + m)
        return cls._instance


    @classmethod
    def instance(cls) -> Device:
        """
        Return the device instance
        """
        return cls._instance
