#!/usr/bin/python3

import logging
import syslog
from os import times
import threading
import serial
import time
import sys
import subprocess
from uci import Uci

serialPort = "/dev/ttyACM0"
tty = None

from uci import Uci

def uciGet( uci: Uci, config: str, section: str, opt: str) -> str:
    try:
        return uci.get(config,section,opt)
    except:
        return ""

def uciSet( uci:Uci, config:str, section:str, opt:str, value:str ) -> None:
    try:
        uci.set(config,section,opt,value)
        uci.commit(config)
    except:
        pass

def openSerial():
    global tty

    tty = serial.Serial()
    tty.port = serialPort
    tty.baudrate = 115200
    tty.bytesize = serial.EIGHTBITS
    tty.parity = serial.PARITY_NONE
    tty.stopbits = serial.STOPBITS_ONE
    tty.xonxoff    = False
    tty.rtscts     = False
    tty.dsrdtr     = False
    tty.rs485_mode = None
    tty.timeout    = 0.2
    tty.open()

def runInBG(stop):
    while not stop:
        tty.write("#IIA\r\n".encode("utf-8"))

        time.sleep(1)

def waitForGD() -> bool:
    timeout = 0
    while True:
        # print(".", end="", flush=True)
        tty.write("#SPC:50:0:0:0\r\n".encode("utf-8"))

        avail = tty.in_waiting
        if avail>0:
            # print("")
            return True

        timeout = timeout + 1
        if timeout > 40:        # 2 sec
            # print("")
            return False

        time.sleep(0.05)

def readCurrentValue() -> str:
    timeout = 0
    tty.reset_input_buffer()
    while True:
        avail = 0
        avail = tty.in_waiting
        if avail>0:
            try:
                data = tty.read(avail)
                current = data.decode()
                current = current.split("\n")[0]
                if current[0:4] == "#CPL":
                    return current
            except Exception as ex:
                pass

        timeout = timeout + 1
        if timeout > 200:        # 1 sec
            return False

        time.sleep(0.005)

def readCurrentQ(n)->float:
    r=3
    while r>1.5:
        data = []
        for m in range(0, n):
            i = readCurrentValue()
            tmp = i.split(":")
            if len(tmp) != 5:
                continue
            data.append(float(tmp[3]))
        data.sort()
        data = data[:-2]
        print(data)
        r = max(data)/min(data)
    
    print(data)
    
    return round(sum(data)/len(data), 4)

def readCurrentA(n)->float:
    first = True
    r=3
    while r>1.5:
        data = []
        subprocess.Popen(["aplay", "-q", "-D", "plug:rtp_dmix", "/barix/apps/epic-sip/beep-11Hz.wav"])
        time.sleep(0.2)
        for m in range(0, n):
            i = readCurrentValue()
            tmp = i.split(":")
            if len(tmp) != 5:
                continue
            data.append(float(tmp[3]))
        data.sort()
        data = data[2:]
        print(data)
        r = max(data)/min(data)
        time.sleep(2)
        
    print(data)
    
    return round(sum(data)/len(data), 4)


if __name__ == "__main__":
    print("Opening serial port")
    openSerial()
    print("Serial port opened")

    tty.reset_output_buffer()
    
    print("Waiting for GD to come alive")
    if not waitForGD():
        print("GD never got ready")
        tty.close()
        sys.exit(1)
    print("GD ready")

    # Start the heartbeat
    print("Starting heartbeat thread")
    stopHB = False
    th = threading.Thread(target=runInBG, args=[lambda: stopHB])
    th.start()

    # Mute the amplifier
    #tty.write("#SAM:1\r\n".encode("utf-8"))

    # Set o/p volume to -6db, preserving the existing configuration
    #curr_vol = uciGet(Uci(),"ae_ms", "volume", "out0")
    #uciSet(Uci(),"ae_ms", "volume", "out0", "-12")
    #uciSet(Uci(),"ae_ms", "speaker", "impedance", "8")  # To avoid clipping by the dsp app
    #rslt = subprocess.call(["barix-wd", "--pid-file=/var/run/ms700-dsp.pid", "--restart"])
    #time.sleep(1)

    # Un-mute the amplifier, set to max level and disable the peripherals
    #tty.write("#SAM:1\r\n".encode("utf-8"))
    
    # Measure the consumption with the peripherals enabled 
    #tty.write("#SOL:16:1\r\n".encode("utf-8"))
    #time.sleep(1)
    r=3
    while r>1.5:
        print("Reading the quiescent consumption with the amp muted and the peripherals ON")
        Ip1 = readCurrentQ(13)
        Ip2 = readCurrentQ(13)
        r = max(Ip1, Ip2)/min(Ip1, Ip2)
    Ip=round((Ip1 + Ip2)/2, 4)
    with open("/tmp/consumption", "w") as f:
        f.write("Ip={}\n".format(Ip))
    
    # Re-read the quiescent consumption with the amp enabled
    #tty.write("#SAM:0\r\n".encode("utf-8"))
    #tty.write("#SAL:11\r\n".encode("utf-8"))
    #r=3
    #while r>1.5:
    #    print("Reading the quiescent consumption")
    #    Iq1 = readCurrentQ(13)
    #    Iq2 = readCurrentQ(13)
    #    r = max(Iq1, Iq2)/min(Iq1, Iq2)
    #Iq=round((Iq1 + Iq2)/2, 4)
    
    #r=3
    #while r>1.5:
    #    print("Reading the active consumption")
    #    Ia1 = readCurrentA(13)
    #    Ia2 = readCurrentA(13)
    #    r = max(Ia1, Ia2)/min(Ia1, Ia2)
    #Ia=round((Ia1 + Ia2)/2, 4)

    # Store the measurements with 
    #diff=round(Ia-Iq, 4)
    #print("Clean diff1: ({}, {}) {}".format(Ia, Iq, diff))
    
    #si = 4
    #if diff<0.0877:
    #    si = 8
    #elif diff>0.1566:
    #    si = 2
    #print("Speaker impedance: {} Ohm".format(si))

    #print("\nRestoring environment")
    
    # Configure speaker
    #uciSet(Uci(),"ae_ms", "speaker", "impedance", str(si))

    # Restore volume configuration
    #uciSet(Uci(),"ae_ms", "volume", "out0", curr_vol)

    # Mute the amplifier
    #tty.write("#SAM:1\r\n".encode("utf-8"))
    
    # Re-enable the peripherals
    #tty.write("#SOL:16:1\r\n".encode("utf-8"))
    
    # Restart dsp to apply the changes
    #rslt = subprocess.call(["barix-wd", "--pid-file=/var/run/ms700-dsp.pid", "--restart"])

    # Unmute the amplifier
    #tty.write("#SAM:0\r\n".encode("utf-8"))

    # Stop heartbeat thread
    stopHB = True
    th.join()
    th = None
    
    tty.close()

