
import threading

from barix.io.Gpio import Gpio

class InOut:
    IDLE     = 0
    PRESSED  = 1
    RELEASED = 2

    def __init__(self, pin:int ):
        self.lock   = threading.Lock()
        self.mode   = Gpio.IN
        self.outVal = None
        self.gpio   = Gpio(str(pin),"in")
        self.state  = self.gpio.read()
        self.relayOn= False


    def checkEvents(self) -> int:
        """
        Check for updates on the input
        """
        try:
            self.lock.acquire()
            if self.relayOn:
                return InOut.IDLE

            stt = self.gpio.read()
            if self.state==stt:
                return InOut.IDLE

            self.state  = stt
            if stt==1:
                return InOut.RELEASED
            return InOut.PRESSED
        finally:
            self.lock.release()


    def read(self) -> int:
        """
        Read the input. Does not update event tracking.
        """
        try:
            self.lock.acquire()
            if self.relayOn:
                return 1

            return self.gpio.read()
        finally:
            self.lock.release()


    def setRelay(self,value: bool) -> None:
        try:
            self.lock.acquire()
            if value:
                if not self.relayOn:
                    self.gpio.setDirection(Gpio.OUT)
                    self.gpio.write(0)
                    self.relayOn    = True
            else:
                self.gpio.setDirection(Gpio.IN)
                self.relayOn    = False
        finally:
            self.lock.release()

    def getMode(self) -> str:
        if self.relayOn:
            return "RELAY"
        return "INPUT"