
import logging
import alsaaudio
import subprocess

from uci import Uci

from barix.io import leds
from device import Device

from inout import InOut

def uciGet( uci: Uci, config: str, section: str, opt: str) -> str:
    try:
        return uci.get(config,section,opt)
    except:
        return None


class DeviceMs500(Device):
    """
    Device adapter for MS-500A and MS-300B
    """

    def shutdown(self):
        pass

    def __init__(self,model):
        Device.__init__(self,model)
        self.logger     = logging.getLogger("device")
        uci             = Uci()
        serialGwMode    = uciGet(uci,"epic","serialgw","mode")
        serialBaud      = int(uciGet(uci,"epic","serialgw","baud"))
        serialDatabits  = int(uciGet(uci,"epic","serialgw","databits"))
        serialParity    = uciGet(uci,"epic","serialgw","parity")
        serialStopBits  = int(uciGet(uci,"epic","serialgw","stopbits"))
        serialPeerPort  = int(uciGet(uci,"epic","serialgw","port"))
        serialPeerPort  = int(uciGet(uci,"epic","serialgw","port"))
        serialPeerHost  = uciGet(uci,"epic","serialgw","host")
        if serialGwMode=="server":
            self.serialGw   = SerialGw("/dev/ttyS1",SerialGw.SERVER)
        elif serialGwMode=="disabled":
            self.serialGw   = SerialGw("/dev/ttyS1",SerialGw.NOGW)
        else:
            self.logger.error("unknown serialgw mode: {}".format(serialGwMode))

            self.serialGw.setSerialParams(serialBaud,serialDatabits,serialParity,serialStopBits)
            self.serialGw.setPeer(serialPeerPort,serialPeerHost)
            self.serialGw.start()
            # self.device.setSerialGw(self.serialGw) #????
            logMsg      = "Loaded serial to TCP gateway: mode='{}'".format(serialGwMode)
            self.logger.info(logMsg)
            syslog.syslog(logMsg)

        self.io1        = InOut(10)
        self.io2        = InOut(13)
        self.redLed     = leds.Led("barix:led1:red")
        self.greenLed   = leds.Led("barix:led1:green")

    def getIo(self,id:int) -> InOut:
        if id==1:
            return self.io1
        elif id==2:
            return self.io2
        else:
            raise Exception("Unknown IO: {}".format(id))

    def setRelay(self, relayId:int, stt:bool) -> None:
        self.getIo(relayId).setRelay(stt)

    def setRedLed(self,stt:bool) -> None:
        self.redLed.set(stt)

    def setGreenLed(self,stt:bool) -> None:
        self.greenLed.set(stt)

    def readInEvent(self, pinId:int) -> int:
        return self.getIo(pinId).checkEvents()

    def readInValue(self, pinId:int) -> int:
        return self.getIo(pinId).read()

    def getIoMode(self, pinId:int) -> str:
        return self.getIo(pinId).getMode()

    def teardown(self):
        self.serialGw.shutdown()

    def getSipPcmc(self) -> str:
        """
        Get the name of the SIP PCM capture device (microphone)
        """
        return "plug:dsnoop_analog"


    def getSipPcmp(self) -> str:
        """
        Get the name of the SIP PCM playback device (audio out)
        """
        return "chan_3"


    def getModel(self) -> str:
        """
        Get the device model
        """
        return self._model

    def getRtpBgmCh(self) -> str:
        """
        Get the alsa channel for RTP BGM
        """
        return "chan_1"

    def getRtpHpnCh(self) -> str:
        """
        Get the alsa channel for RTP HPN
        """
        return "chan_2"

    def loadMixers(self) -> None:
        self._setMixerParam("Line In", "cap")
        self._setMixerParam("ADC Gain", "0dB")
        self._setMixerParam("Mic1", "nocap")
        alsaaudio.PCM(device='channel1')
        alsaaudio.PCM(device='channel2')
        alsaaudio.PCM(device='channel3')
        alsaaudio.PCM(device='channel4')
        alsaaudio.PCM(device='channel5')

    def setExclusiveInput( self , id: int ) -> None:
        if id<0 or id>6:
            raise Exception("Invalid input specifier {}".format(id))
        self.serialGw.writeSideChannel("$SXI:{}\r\n".format(id))

    def _setMixerParam(self, param, value):
        p = subprocess.Popen("amixer -q -c 2 -- sset " + "\'" + param + "\'" + " \'" + value + "\'", shell=True)
        p.wait()
