#!/bin/sh

TARGET="/barix/PoE_type.txt"
LOG_FILE="/var/log/poe_identification.log"

# Create gpio14 device if does not exist
if [ ! -d "/sys/class/gpio/gpio14" ]; then
    echo 14 > /sys/class/gpio/export
fi
echo in >  /sys/class/gpio/gpio14/direction

echo $(date +%Y-%m-%d_%H:%M:%S) > $LOG_FILE

# Read the hardware signal
HW_SIGNAL=$( cat /sys/class/gpio/gpio14/value )

# Check if HW signal declares a 'type 2' switch
if [ "$HW_SIGNAL" = "0" ]; then
    echo "type=2" > $TARGET
    echo "HW_SIGNAL=$HW_SIGNAL"
    echo "PoE+ successefully identified by hardware." >> $LOG_FILE
    exit 0
fi

# DLL negotiation
# It usually takes more than 5 sec to respond
echo "Starting lldp negotiation"
TYPE=0
COUNT1=0
COUNT2=0
while true; do
	REPLY=$( lldpctl -f json )
	echo $REPLY >> $LOG_FILE
    POWER=$( echo $REPLY | jq -r '.lldp.interface.eth0.port.power' )
    
    # Wait for a valid response in the case of a slow switch
    if [ "$POWER" = "null" ]; then
        COUNT2=$((COUNT2+1))
		if [[ $COUNT2 -gt 5 ]]; then
		    echo "Loop1: lldp negotiation timeout."
			break
		fi
		sleep 6
		continue
	fi
	
    POWER_TYPE=$( echo "$POWER" | jq -r '."power-type"' )
    echo "Switch json reply: $REPLY"
    echo "Reply power section: $POWER"
    echo "Power type value: $POWER_TYPE"
	if [ "x$POWER_TYPE" = "x" ]; then
		COUNT1=$((COUNT1+1))
		if [[ $COUNT1 -gt 30 ]]; then
		    echo "Loop2: lldp negotiation timeout."
			break
		fi
	else
        if [ "$POWER_TYPE" = "2" ]; then
            echo "Power type determined through lldp negotiation."
            TYPE=2
            echo "POWER_TYPE=$TYPE"
        else
            REQUESTED=$( echo "$POWER" | jq -r '.requested' )
            ALLOCATED=$( echo "$POWER" | jq -r '.allocated' )
            if (( ALLOCATED != null )); then
                if (( ALLOCATED >= REQUESTED )); then
                    TYPE=2
                    echo "ALLOCATED>=REQUESTED: $ALLOCATED>=$REQUESTED"
                fi
            fi
        fi
		
		break
	fi
	
	sleep 1
done

# Check if PoE detect overridden by Arcanix
if [ ! "$TYPE" = "2" ]; then
    OVR=$( uci get ae_ms.poe.override )
    
    if [ "$OVR" = "true" ]; then
        TYPE=3
        echo "PoE overridden"
    fi
fi

echo "type=$TYPE" > $TARGET
